package de.fhdw.wtf.common.ast.type.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;

import org.junit.Ignore;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException;
import de.fhdw.wtf.tooling.SyntaxCheck;

/**
 * Contains testcases for symmetric attributes.
 */
public class SymmetricTest {
	/**
	 * Tests a several invalid groups.
	 */
	@Ignore
	@Test
	public void testSimpleSymmetricBasic1() {
		final List<String> inputs = new ArrayList<String>();
		final String input1 = "Group:group=[A:class={b:B symmetric(a);};B:class={a:A symmetric(b);};];";
		try {
			final Model model2 = SyntaxCheck.getInstance().getModelFromString(input1);
			final Group group2 = model2.getGroups().iterator().next();
			final Iterator<GroupElement> i = group2.getGroupElements().iterator();
			final ClassType classA = (ClassType) i.next();
			final ClassType classB = (ClassType) i.next();
			final Attribute bInA = classA.getAttributes().iterator().next();
			final Attribute aInB = classB.getAttributes().iterator().next();
			assertEquals("b", bInA.getName());
			assertEquals("a", aInB.getName());
			assertEquals(true, bInA.isSymmetric());
			assertEquals(true, aInB.isSymmetric());
			assertEquals(aInB.getAttrType(), classA);
			assertEquals(bInA.getAttrType(), classB);
			
		} catch (final Exception e) {
			fail();
		}
		final String input2 = "Group:group=[A:class={b:B* symmetric(a);};B:class={a:A* symmetric(b);};];";
		final String input3 = "Group:group=[A:class={b:B* symmetric(a);};B:class={a:A symmetric(b);};];";
		final String input4 = "Group:group=[A:class={a1:A* symmetric(a2);a2:A* symmetric(a1);};];";
		final String input5 =
				"Group:group=[A:class={b: group2.B symmetric(a);};];"
						+ "Group:group2=[B:class={a: group.A symmetric(b);};];";
		final String input6 =
				"Group:group=[A:class={b:B symmetric(a);b2:B symmetric(a2);};"
						+ "B:class={a:A symmetric(b);};a2:A symmetric(b2);};];";
		inputs.add(input1);
		inputs.add(input2);
		inputs.add(input3);
		inputs.add(input4);
		inputs.add(input5);
		inputs.add(input6);
		for (final String input : inputs) {
			try {
				SyntaxCheck.getInstance().getModelFromString(input);
			} catch (final Exception e) {
				fail();
			}
		}
		Model model3;
		try {
			model3 = SyntaxCheck.getInstance().getModelFromString(input6);
			final Group group3 = model3.getGroups().iterator().next();
			final Iterator<GroupElement> i = group3.getGroupElements().iterator();
			final ClassType classA = (ClassType) i.next();
			final ClassType classB = (ClassType) i.next();
			assertEquals(2, classA.getAttributes().size());
			assertEquals(2, classB.getAttributes().size());
		} catch (MultipleCheckExceptions | CyclicDependencyException | InterruptedException
				| CyclicPartDefinitionException | ExecutionException e) {
			fail();
		}
	}
	
	/**
	 * Tests several invalid groups.
	 */
	@Test
	public void testSimpleSymmetricBasic2() {
		final List<String> inputs = new ArrayList<String>();
		final String input1 = "Group:group=[A:class={b:C symmetric(a);};B:class={a:A symmetric(b);};C:class={};];";
		final String input2 = "Group:group=[A:class={b:B symmetric(a);};B:class={a:C symmetric(b);};C:class={};];";
		final String input3 =
				"Group:group=[A:class={b:C symmetric(a);};B:class={a:A symmetric(b);};C:class={a:A symmetric};];";
		final String input4 =
				"Group:group=[A:class={b:B symmetric(c);};B:class={c:C symmetric(d);};"
						+ "C:class={d:D symmetric(a);};D:class={a:A symmetric(B);};];";
		final String input5 = "Group:group=[A:class={b:B symmetric(a);};B:class={a:A ;};];";
		final String input6 = "Group:group=[A:class={a:A symmetric(a);};];";
		inputs.add(input1);
		inputs.add(input2);
		inputs.add(input3);
		inputs.add(input4);
		inputs.add(input5);
		inputs.add(input6);
		for (final String input : inputs) {
			try {
				SyntaxCheck.getInstance().getModelFromString(input);
				fail();
			} catch (final Exception e) {
				// mistake found
			}
		}
	}
}
