package de.fhdw.wtf.core.integration.testutil;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.Collection;
import java.util.Iterator;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.type.AtomicType;
import de.fhdw.wtf.common.ast.type.ByReferenceState;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.CompositeType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.ThrownType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitor;
import de.fhdw.wtf.common.ast.visitor.GroupElementVisitor;
import de.fhdw.wtf.common.ast.visitor.TypeVisitor;
import de.fhdw.wtf.walker.walker.HelperUtils;

@SuppressWarnings("javadoc")
public class AssertOperations {
	
	public void assertAllReferencesByType(final Model model) {
		final Iterator<Group> modelGroups = model.getGroups().iterator();
		while (modelGroups.hasNext()) {
			final Group current = modelGroups.next();
			this.assertByTypeInGroup(current);
		}
	}
	
	private void assertByTypeInGroup(final Group group) {
		final Iterator<GroupElement> elements = group.getGroupElements().iterator();
		while (elements.hasNext()) {
			final GroupElement current = elements.next();
			current.accept(new GroupElementVisitor() {
				@Override
				public void handle(final ClassType clss) {
					AssertOperations.this.assertByTypeInClass(clss);
				}
				
				@Override
				public void handle(final Group group) {
					AssertOperations.this.assertByTypeInGroup(group);
				}
			});
		}
	}
	
	private void assertByTypeInClass(final ClassType clazz) {
		this.assertByTypeSuperTypes(clazz.getSuperTypes());
		this.assertByTypeAttributes(clazz.getAttributes());
		this.assertByTypeOperations(clazz.getOperations());
	}
	
	private void assertByTypeSuperTypes(final Collection<Type> supertypes) {
		for (final Type supertype : supertypes) {
			try {
				HelperUtils.getReferencedType(supertype);
			} catch (final Error e) {
				fail("The State is no ByReferenceState");
			}
		}
	}
	
	private void assertByTypeAttributes(final Collection<Attribute> attributes) {
		for (final Attribute attribute : attributes) {
			final Type current = attribute.getAttrType();
			this.assertByTypeType(current);
		}
	}
	
	private void assertByTypeOperations(final Collection<Operation> operations) {
		for (final Operation operation : operations) {
			final Type returnValue = operation.getReturnType();
			this.assertByTypeType(returnValue);
			final ProductType params = operation.getSignature().getParameters();
			this.assertByTypeType(params);
		}
	}
	
	private void assertByTypeType(final Type type) {
		type.accept(new TypeVisitor() {
			
			@Override
			public void handle(final TypeProxy s) {
				assertEquals(true, s.getState() instanceof ByReferenceState);
			}
			
			@Override
			public void handle(final CompositeType c) {
				c.accept(new CompositeTypeVisitor() {
					
					@Override
					public void handle(final SumType sum) {
						for (final Type sumElement : sum.getElements()) {
							AssertOperations.this.assertByTypeType(sumElement);
						}
					}
					
					@Override
					public void handle(final ProductType product) {
						for (final ProductElementType productElement : product.getElements()) {
							AssertOperations.this.assertByTypeType(productElement.getType());
						}
					}
					
					@Override
					public void handle(final MapType map) {
						AssertOperations.this.assertByTypeType(map.getKey());
						AssertOperations.this.assertByTypeType(map.getOf());
					}
					
					@Override
					public void handle(final ListType list) {
						AssertOperations.this.assertByTypeType(list.getOf());
					}
					
					@Override
					public void handle(final ThrownType thrownType) {
						AssertOperations.this.assertByTypeType(thrownType.getUnderlyingType());
					}
				});
			}
			
			@Override
			public void handle(final AtomicType s) {
				fail();
				// Kommt nie
			}
		});
	}
	
}
