package de.fhdw.wtf.walker.tasks.test;

import static org.junit.Assert.assertEquals;

import java.util.concurrent.ExecutionException;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.core.integration.testutil.AssertOperations;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.util.ConstructorReferencerTestReturnValue;

/**
 * Test the functions of the referencer: - Finding the right constructor for a given name in {@link Model}. - Create
 * anything.
 */
public class TestConstructorReferencer {
	
	/**
	 * Contains different functions to check the results of referencing.
	 */
	private AssertOperations assertOperations;
	
	/**
	 * Sets up the {@link AssertOperations} for each testcase.
	 */
	@Before
	public void setUp() {
		this.assertOperations = new AssertOperations();
	}
	
	/**
	 * Test the referencing of:Group:group=[ A:class={ (x:Integer); }; B:class=A+{ ()=A(Integer); }; ];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSuperConstructorWorking() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={(x:Integer);};B:class=A+{()=A(Integer);};];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final ConstructorReferencerTestReturnValue v = new ConstructorReferencerTestReturnValue(result);
		
		assertEquals(0, v.getFailResult().size());
		assertEquals(2, v.getOkResult().size());
	}
	
	/**
	 * Test the referencing of:Group:group=[ A:class={ (x:Integer); }; B:class=A+{ ()=A(String); }; ];.
	 * 
	 * @throws Exception
	 *             {@link de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException}, {@link InterruptedException}
	 *             , {@link ExecutionException}.
	 */
	@Test
	public void testSuperConstructorFailing() throws Exception {
		final Scanner scanner = ModelDslScanner.create();
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[A:class={(x:Integer);};B:class=A+{()=A(String);};];");
		final TokenStream output = FilteredTokenStream.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model result = parser.parse();
		
		final ConstructorReferencerTestReturnValue v = new ConstructorReferencerTestReturnValue(result);
		
		assertEquals(1, v.getFailResult().size());
		assertEquals(1, v.getOkResult().size());
	}
	
}
